@@ -70,15 +70,27 @@ def upload_s3(bucket: str, key: str, source: Union[str, BytesIO]) -> None:
         client.upload_fileobj(Fileobj=source, Bucket=bucket, Key=key)
 
 
-def _get_s3_keys(bucket: str, prefix: str = '') -> List[str]:
+def _get_s3_keys(bucket: str, prefix: str = '') -> Generator[str, None, None]:
     object_list = get_client().list_objects_v2(Bucket=bucket, Prefix=prefix)
 
     if 'Contents' not in object_list:
-        return []
+        return
 
-    keys = [file['Key'] for file in object_list['Contents']]
+    for file in object_list['Contents']:
+        yield file['Key']
 
-    return keys
+    while not object_list['IsTruncated']:
+        object_list = get_client().list_objects_v2(
+            Bucket=bucket,
+            Prefix=prefix,
+            ContinuationToken=object_list['ContinuationToken'],
+        )
+
+        if 'Contents' not in object_list:
+            return
+
+        for file in object_list['Contents']:
+            yield file['Key']
 
 
 def _get_local_files(path: str) -> List[str]:
@@ -107,7 +119,7 @@ def exists(path: str) -> bool:
         return os.path.exists(path)
 
     bucket, key = get_bucket_and_key(path)
-    has_keys = len(_get_s3_keys(bucket, key)) > 0
+    has_keys = sum(1 for _ in _get_s3_keys(bucket, key)) > 0
 
     return has_keys
 
@@ -130,16 +142,14 @@ def localize_directory(
         overwrite: bool = True,
 ) -> str:
 
-    path_prefixed = os.path.join(path, prefix) if prefix else path
-
     if not path.startswith('s3'):
-        return path_prefixed
+        return path
 
     directory = generate_path(os.path.basename(path)) if directory is None else directory
-    directory_prefixed = os.path.join(directory, prefix) if prefix else directory
 
+    directory_prefixed = os.path.join(directory, prefix) if prefix else directory
     if not overwrite and os.path.exists(directory_prefixed):
-        return directory_prefixed
+        return directory
 
     bucket, key_base = get_bucket_and_key(path)
     key_base_prefixed = os.path.join(key_base, prefix) if prefix else key_base
@@ -231,7 +241,7 @@ def save_npy(path: str, data: np.ndarray) -> None:
         np.save(path, data)
 
     else:
-        with open(path, mode='wb') as writer:
+        with io.open(path, mode='wb') as writer:
             np.save(writer, data)
 
 
